@@ -12,22 +12,22 @@ import itertools
 import os
 import shlex
 
-__version__ = '0.3'
+__version__ = '0.4'
 
 
 #---
-def read(nml_fname, assume_kind_type=False, verbose=False):
+def read(nml_fname, verbose=False):
     """Parse a Fortran 90 namelist file and store the contents in a ``dict``.
 
     >>> data_nml = f90nml.read('data.nml')"""
 
     nml_file = open(nml_fname, 'r')
 
-    f90 = shlex.shlex(nml_file)
-    f90.commenters = '!'
-    f90.escapedquotes = '\'"'
-    f90.wordchars += '.-+'      # Include floating point characters
-    tokens = iter(f90)
+    f90lex = shlex.shlex(nml_file)
+    f90lex.commenters = '!'
+    f90lex.escapedquotes = '\'"'
+    f90lex.wordchars += '.-+'      # Include floating point characters
+    tokens = iter(f90lex)
 
     # Store groups in case-insensitive dictionary
     nmls = NmlDict()
@@ -36,10 +36,11 @@ def read(nml_fname, assume_kind_type=False, verbose=False):
 
         # Ignore tokens outside of namelist groups
         while t != '&':
-            t = next(tokens)
+            t, prior_t = next(tokens), t
 
         # Current token is now '&'
 
+        # Create the next namelist
         g_name = next(tokens)
         g_vars = NmlDict()
 
@@ -47,47 +48,52 @@ def read(nml_fname, assume_kind_type=False, verbose=False):
         v_idx = None
         v_vals = []
 
-        prior_t = t
-        t = next(tokens)
-
         # Current token is either a variable name or finalizer (/, &)
 
-        while not t in ('/', '&'):
+        # Populate the namelist group
+        while g_name:
+
+            if not v_name:
+                t, prior_t = next(tokens), t
+
+                # Skip commas separating objects
+                if t == ',':
+                    t, prior_t = next(tokens), t
+                # Parse the indices of the current variable
+            else:
+                if t == '(':
+                    v_name, v_indices, t = parse_f90idx(tokens, t, prior_t)
+
+                    # TODO: Multidimensional support
+                    # TODO: End index support (currently ignored)
+                    i_s = 1 if not v_indices[0][0] else v_indices[0][0]
+                    i_r = 1 if not v_indices[0][2] else v_indices[0][2]
+                    v_idx = itertools.count(i_s, i_r)
 
             # Diagnostic testing
             if verbose:
-                print('state: {} {}'.format(t, prior_t))
+                print('  tokens: {} {}'.format(prior_t, t))
+
+            while v_name:
 
-            # Skip commas
-            if t == ',':
                 t, prior_t = next(tokens), t
 
-            # Advance token
-            if not t in ('/', '&'):
-                t, prior_t = next(tokens), t
+                if verbose:
+                    print('    vstate: {} {} {}'.format(v_name, v_idx, v_vals))
+                    print('    tokens: {} {}'.format(prior_t, t))
 
-            if v_name:
-
-                #---
                 # Parse the prior token value
-
-                # Skip variable names and end commas
-                if not (t == '=' or (t == '(' and not prior_t == '=')
-                        or (prior_t, t) == (',', '/')
-                        or (prior_t, t) == ('=', '/')
-                        or (prior_t, t) == (',', '&')
-                        or prior_t == ')'):
-
-                    # Skip ahead on first value
-                    if prior_t == '=':
-                        t, prior_t = next(tokens), t
-
+                # TODO: Add '%' to first tuple
+                if (not t in ('(', '=') or prior_t == '=') \
+                        and not (prior_t, t) == (',', '/'):
                     # Parse the variable string
-                    if prior_t == ',':
-                        next_value = None
+                    if prior_t in ('=', ','):
+                        if t in (',', '/', '&'):
+                            next_value = None
+                        else:
+                            continue
                     else:
-                        next_value, t = parse_f90val(tokens, t, prior_t,
-                                                     assume_kind_type)
+                        next_value, t = parse_f90val(tokens, t, prior_t)
 
                     if v_idx:
 
@@ -110,6 +116,7 @@ def read(nml_fname, assume_kind_type=False, verbose=False):
                         v_vals.append(next_value)
 
                 # Save then deactivate the current variable
+                # TODO: Add '%'
                 if t in ('(', '=', '/', '&'):
 
                     if len(v_vals) == 0:
@@ -122,29 +129,22 @@ def read(nml_fname, assume_kind_type=False, verbose=False):
                     v_name = None
                     v_vals = []
 
-            # Parse the indices of the current variable
-            if t == '(' and not prior_t == '=':
-                v_name, v_indices, t = parse_f90idx(tokens, t, prior_t)
-
-                # TODO: Multidimensional support
-                # TODO: End index support (currently ignored)
-                i_s = 1 if not v_indices[0][0] else v_indices[0][0]
-                i_r = 1 if not v_indices[0][2] else v_indices[0][2]
-                v_idx = itertools.count(i_s, i_r)
-
             # Set the next active variable
-            if t == '=' and not v_name:
+            # TODO: Add '%' to the list
+            if t in ('=', '('):
                 v_name = prior_t
                 v_idx = None
 
-            # Test for classic namelist termination
-            if t == '&':
-                t, prior_t = next(tokens), t
-                if t == 'end':
-                    break
+            # Finalise namelist group
+            if t in ('/', '&'):
+                # Test for classic namelist finaliser
+                if t == '&':
+                    t, prior_t = next(tokens), t
+                    assert t.lower() == 'end'
 
-        # Append the grouplist to the namelist (including empty groups)
-        nmls[g_name] = g_vars
+                # Append the grouplist to the namelist (including empty groups)
+                nmls[g_name] = g_vars
+                g_name, g_vars = None, None
 
     nml_file.close()
 
@@ -202,7 +202,7 @@ def to_f90str(value):
 
 
 #---
-def parse_f90val(tokens, t, s, assume_kind_type=False):
+def parse_f90val(tokens, t, s):
     """Convert string repr of Fortran type to equivalent Python type."""
     assert type(s) is str
 
@@ -212,6 +212,10 @@ def parse_f90val(tokens, t, s, assume_kind_type=False):
         next(tokens)
         s_im = next(tokens)
 
+        # Bypass the right parenthesis
+        t = next(tokens)
+        assert t == ')'
+
         t = next(tokens)
 
         s = '({}, {})'.format(s_re, s_im)
@@ -220,12 +224,7 @@ def parse_f90val(tokens, t, s, assume_kind_type=False):
 
     for f90type in recast_funcs:
         try:
-            if assume_kind_type and f90type == int:
-                value = f90type(s.rpartition('_')[0])
-            elif assume_kind_type and f90type in (f90float, f90complex):
-                value = f90type(s, assume_kind_type)
-            else:
-                value = f90type(s)
+            value = f90type(s)
             return value, t
         except ValueError:
             continue
@@ -235,17 +234,14 @@ def parse_f90val(tokens, t, s, assume_kind_type=False):
 
 
 #---
-def f90float(s, assume_kind_type=False):
+def f90float(s):
     """Convert string repr of Fortran floating point to Python double"""
 
-    if assume_kind_type:
-        s, _, _ = s.rpartition('_')
-
     return float(s.lower().replace('d', 'e'))
 
 
 #---
-def f90complex(s, assume_kind_type=False):
+def f90complex(s):
     """Convert string repr of Fortran complex to Python complex."""
     assert type(s) == str
 
@@ -253,8 +249,7 @@ def f90complex(s, assume_kind_type=False):
         s_re, s_im = s[1:-1].split(',', 1)
 
         # NOTE: Failed float(str) will raise ValueError
-        return complex(f90float(s_re, assume_kind_type),
-                       f90float(s_im, assume_kind_type))
+        return complex(f90float(s_re), f90float(s_im))
     else:
         raise ValueError('{} must be in complex number form (x, y)'.format(s))
 
