@@ -10,7 +10,9 @@ class Test(unittest.TestCase):
         self.empty_nml = {'empty_nml': {}}
 
         self.null_nml = {'null_nml':
-                            {'null_value': None}
+                            {'null_value': None},
+                         'null_comma_nml':
+                            {'null_comma': None}
                         }
 
         self.types_nml = {'types_nml':
@@ -49,13 +51,6 @@ class Test(unittest.TestCase):
                             }
                          }
 
-        self.kind_nml = {'kind_nml':
-                            {'int_kind': 123,
-                             'real_kind': 1.0,
-                             'complex_kind': 1. - 2.j
-                            }
-                        }
-
         self.comment_nml = {'comment_nml':
                             {'v_cmt_inline': 123,
                              'v_cmt_in_str': 'This token ! is not a comment',
@@ -90,10 +85,6 @@ class Test(unittest.TestCase):
         test_nml = f90nml.read('float.nml')
         self.assertEqual(self.float_nml, test_nml)
 
-    def test_kind(self):
-        test_nml = f90nml.read('kind.nml', assume_kind_type=True)
-        self.assertEqual(self.kind_nml, test_nml)
-
     def test_comment(self):
         test_nml = f90nml.read('comment.nml')
         self.assertEqual(self.comment_nml, test_nml)
